@@ -23,7 +23,7 @@
 
 // Increased any time the API changes. (Note: Does not reset when PL_MAJOR_VER
 // is increased)
-#define PL_API_VER 157
+#define PL_API_VER 192
 
 // Increased any time a fix is made to a given API version.
 #define PL_FIX_VER (pl_fix_ver())
@@ -52,4 +52,21 @@ const char *pl_version(void);
 #define PL_DEPRECATED __attribute__((deprecated))
 #endif
 
+// C++ compatibility
+#ifdef __cplusplus
+# define PL_STRUCT(name) struct name##_t
+# define PL_API_BEGIN extern "C" {
+# define PL_API_END }
+#else
+# define PL_STRUCT(name) struct name
+# define PL_API_BEGIN
+# define PL_API_END
+// Disable this warning because libplacebo's params macros override fields
+# pragma GCC diagnostic ignored "-Woverride-init"
+#endif
+
+// Extra helper macros
+#define PL_TOSTRING_INNER(x) #x
+#define PL_TOSTRING(x) PL_TOSTRING_INNER(x)
+
 #endif // LIBPLACEBO_CONTEXT_H_
