@@ -22,8 +22,8 @@
 #include <libplacebo/colorspace.h>
 #include <libplacebo/filters.h>
 #include <libplacebo/gpu.h>
-#include <libplacebo/shaders/av1.h>
 #include <libplacebo/shaders/colorspace.h>
+#include <libplacebo/shaders/film_grain.h>
 #include <libplacebo/shaders/lut.h>
 #include <libplacebo/shaders/sampling.h>
 #include <libplacebo/shaders/custom.h>
@@ -47,6 +47,8 @@ size_t pl_renderer_save(pl_renderer rr, uint8_t *out_cache);
 
 // Load the result of a previous `pl_renderer_save` call. See
 // `pl_dispatch_load` for more information.
+//
+// Note: See the security warnings on `pl_pass_params.cached_program`.
 void pl_renderer_load(pl_renderer rr, const uint8_t *cache);
 
 enum pl_lut_type {
@@ -115,8 +117,9 @@ struct pl_render_params {
     // used for 1D mixing and thus only 1D filters are compatible.
     //
     // If set to NULL, frame mixing is disabled, in which case
-    // `pl_render_image_mix` behaves as `pl_render_image`, also completely
-    // bypassing the mixing cache.
+    // `pl_render_image_mix` will use nearest-neighbour semantics. (Note that
+    // this still goes through the redraw cache, unless you also enable
+    // `skip_caching_single_frame`)
     const struct pl_filter_config *frame_mixer;
 
     // Configures the settings used to deband source textures. Leaving this as
@@ -188,6 +191,13 @@ struct pl_render_params {
     float background_transparency; // 0.0 for opaque, 1.0 for fully transparent
     bool skip_target_clearing;
 
+    // If true, then transparent images will made opaque by painting them
+    // against a checkerboard pattern consisting of alternating colors. If both
+    // colors are left as {0}, they default respectively to 93% and 87% gray.
+    bool blend_against_tiles;
+    float tile_colors[2][3];
+    int tile_size;
+
     // --- Performance / quality trade-off options:
     // These should generally be left off where quality is desired, as they can
     // degrade the result quite noticeably; but may be useful for older or
@@ -221,6 +231,12 @@ struct pl_render_params {
     // resize, but should make it much more smooth.
     bool preserve_mixing_cache;
 
+    // Normally, `pl_render_image_mix` will also push single frames through the
+    // mixer cache, in order to speed up re-draws. Enabling this option
+    // disables that logic, causing single frames to bypass the cache. (Though
+    // it will still read from, if they happen to already be cached)
+    bool skip_caching_single_frame;
+
     // --- Performance tuning / debugging options
     // These may affect performance or may make debugging problems easier,
     // but shouldn't have any effect on the quality.
@@ -280,21 +296,35 @@ struct pl_render_params {
     bool disable_overlay_sampling PL_DEPRECATED; // no longer used
 };
 
+// Bare minimum parameters, with no features enabled. This is the fastest
+// possible configuration, and should therefore be fine on any system.
+#define PL_RENDER_DEFAULTS                              \
+    /* set a frame mixer for pl_render_image_mix */     \
+    .frame_mixer        = &pl_filter_oversample,        \
+    .color_map_params   = &pl_color_map_default_params, \
+    .lut_entries        = 64,                           \
+    .tile_colors        = {{0.93, 0.93, 0.93},          \
+                           {0.87, 0.87, 0.87}},         \
+    .tile_size          = 32,                           \
+    .polar_cutoff       = 0.001,
+
+#define pl_render_params(...) (&(struct pl_render_params) { PL_RENDER_DEFAULTS __VA_ARGS__ })
+extern const struct pl_render_params pl_render_fast_params;
+
 // This contains the default/recommended options for reasonable image quality,
 // while also not being too terribly slow. All of the *_params structs are
 // defaulted to the corresponding *_default_params, except for deband_params,
-// which is disabled by default.
+// and peak_detect_params, which are both disabled by default.
 //
-// This should be fine on most integrated GPUs, but if it's too slow, consider
-// setting the params to {0} instead, or alternatively setting
-// `pl_render_params.disable_fbos` to true.
+// This should be fine on most integrated GPUs, but if it's too slow,
+// consider using `pl_render_fast_params` instead.
 extern const struct pl_render_params pl_render_default_params;
 
 // This contains a higher quality preset for better image quality at the cost
 // of quite a bit of performance. In addition to the settings implied by
 // `pl_render_default_params`, it sets the upscaler to `pl_filter_ewa_lanczos`,
-// and enables debanding. This should only really be used with a discrete GPU
-// and where maximum image quality is desired.
+// and enables debanding and peak detection. This should only really be used
+// with a discrete GPU and where maximum image quality is desired.
 extern const struct pl_render_params pl_render_high_quality_params;
 
 // Special filter config for the built-in oversampling algorithm. This is an
@@ -449,6 +479,24 @@ struct pl_frame {
     // the aspect ratio, use a dedicated function like pl_rect2df_aspect_copy.
     struct pl_rect2df crop;
 
+    // Logical rotation of the image, with respect to the underlying planes.
+    // For example, if this is PL_ROTATION_90, then the image will be rotated
+    // to the right by 90° when mapping to `crop`. The actual position on-screen
+    // is unaffected, so users should ensure that the (rotated) aspect ratio
+    // matches the source. (Or use a helper like `pl_rect2df_aspect_set_rot`)
+    //
+    // Note: For `target` frames, this corresponds to a rotation of the
+    // display, for `image` frames, this corresponds to a rotation of the
+    // camera.
+    //
+    // So, as an example, target->rotation = PL_ROTATE_90 means the end user
+    // has rotated the display to the right by 90° (meaning rendering will be
+    // rotated 90° to the *left* to compensate), and image->rotation =
+    // PL_ROTATE_90 means the video provider has rotated the camera to the
+    // right by 90° (so rendering will be rotated 90° to the *right* to
+    // compensate).
+    pl_rotation rotation;
+
     // A list of additional overlays to render directly on top of this frame.
     // These overlays will be treated as though they were part of the frame
     // data, and can be used for things like subtitles or on-screen displays.
@@ -471,24 +519,14 @@ struct pl_frame {
     // treated by libplacebo as an oversized chroma plane - i.e. the plane
     // would get sampled as if it was 17.5 pixels wide and 11.5 pixels large.
 
-    // Associated AV1 grain params (see <libplacebo/shaders/av1.h>). This is
-    // entirely optional, the default of {0} corresponds to no extra grain.
+    // Associated film grain data (see <libplacebo/shaders/film_grain.h>).
     //
     // Note: This is ignored for the `target` of `pl_render_image`, since
     // un-applying grain makes little sense.
-    struct pl_av1_grain_data av1_grain;
+    struct pl_film_grain_data film_grain;
 
     // Ignored by libplacebo. May be useful for users.
     void *user_data;
-
-    // Deprecated fields provided merely for backwards compatibility. The
-    // use of these should be discontinued as soon as possible.
-    int width PL_DEPRECATED; // ignored
-    int height PL_DEPRECATED;
-    uint64_t signature PL_DEPRECATED; // ignored
-    pl_tex fbo PL_DEPRECATED; // fallback for `target.planes`
-    struct pl_rect2df src_rect PL_DEPRECATED; // fallback for `image.crop`
-    struct pl_rect2df dst_rect PL_DEPRECATED; // fallback for `target.crop`
 };
 
 // Helper function to infer the chroma location offset for each plane in a
@@ -520,34 +558,6 @@ static inline void pl_frame_clear(pl_gpu gpu, const struct pl_frame *frame,
     pl_frame_clear_rgba(gpu, frame, clear_color_rgba);
 }
 
-// Deprecated aliases, provided for backwards compatibility
-#define pl_image pl_frame
-#define pl_render_target pl_frame
-
-static PL_DEPRECATED inline void pl_image_set_chroma_location(
-        struct pl_frame *frame, enum pl_chroma_location chroma_loc)
-{
-    return pl_frame_set_chroma_location(frame, chroma_loc);
-}
-
-static PL_DEPRECATED inline void pl_render_target_set_chroma_location(
-        struct pl_frame *frame, enum pl_chroma_location chroma_loc)
-{
-    return pl_frame_set_chroma_location(frame, chroma_loc);
-}
-
-static PL_DEPRECATED inline void pl_render_target_from_swapchain(
-        struct pl_frame *out_frame, const struct pl_swapchain_frame *frame)
-{
-    return pl_frame_from_swapchain(out_frame, frame);
-}
-
-static PL_DEPRECATED inline bool pl_render_target_partial(
-        const struct pl_frame *frame)
-{
-    return pl_frame_is_cropped(frame);
-}
-
 // Render a single image to a target using the given parameters. This is
 // fully dynamic, i.e. the params can change at any time. libplacebo will
 // internally detect and flush whatever caches are invalidated as a result of
@@ -596,8 +606,7 @@ struct pl_frame_mix {
     // section below for more information.
     //
     // If the number of frames is 0, this call will be equivalent to
-    // `pl_render_image` with `image == NULL`. If the number of frames is 1,
-    // this is equivalent to `pl_render_image` with that image.
+    // `pl_render_image` with `image == NULL`.
     int num_frames;
 
     // A list of the frames themselves. The frames can have different
